# -*- coding: utf-8 -*-
# 北海層岩見沢市の2017年を対象に、水稲生育期間の水田水温を計算し、日平均・最高・最低水温のデータ（csv形式）と日平均水温のグラフを出力するサンプル
# 移植は6月1日、成熟は9月28日で、発育ステージDVSが等間隔で増加する場合を仮定。水深Dwは期間を通じて一定値50mmを仮定。
# 本スクリプトはAMD_Tools3およびAMD_PaddyWaterTempをimportしています。利用する場合は別途準備をお願いします。
# Hiroyuki Ohnoさんのsample_GetMetData-c.pyを改変して作成しています。（Atsushi Maruyama 2018.02.08）

import AMD_Tools3 as AMD
import numpy as np
from AMD_PaddyWaterTemp import WaterTemp
from datetime import datetime
import matplotlib.pylab as plt
import matplotlib.dates as md


# 基本的な設定
timedomain = [ "2017-06-01", "2017-09-28" ] #計算期間の設定
lalodomain = [ 43.2463, 43.2463, 141.6946, 141.6946]  #緯度経度設定（北海道岩見沢市）
days = len(AMD.timedom(timedomain)) #日数のカウント

# 発育ステージと水深の設定
dvs_bgn = 0  #計算期間の始まりの発育ステージ、0:移植,1:出穂,2:成熟。
dvs_end = 2  #計算期間の終わりの発育ステージ、0:移植,1:出穂,2:成熟。
dvs = np.arange(dvs_bgn, dvs_end, (dvs_end - dvs_bgn)/days)  #days日間の発育ステージをdvs_bgnからdvs_endまで等間隔に与える場合のサンプル
Dw  = [50] * days                                            #days日間の水深に一定値50mmを与える場合のサンプル

#以下のようにリストに個別に数値を与えることも可能（5日間の場合のサンプル）
#dvs = [0.1, 0.2, 0.3, 0.4, 0.5]  #期間中の発育ステージ(期間分のリストであること)、発育ステージの定義は0:移植,1:出穂,2:成熟。
#Dw  = [50, 50, 30, 30, 30]   #期間中の水深(期間分のリストであること)、単位はmm。

#WaterTemp関数による水温データの取得
Tw_mea, Tw_max, Tw_min, tim, lat, lon = WaterTemp(timedomain, lalodomain, dvs, Dw)
Tw_mea_N, Tw_max_N, Tw_min_N, tim, lat, lon = WaterTemp(timedomain, lalodomain, dvs, Dw, ws=2, cli=True)
#laim（最大葉面積指数）, alb(アルベド), ws（風速）は固定値を設定可能。省略した場合はlaim=4.6, alb=0.1, wsは1kmメッシュデータを使用。
#平年値を取得するときにcli=True（または1）として指定する。ただし風速には平年値がないのでwsで設定する必要がある。



Tw_mea = Tw_mea[:,0,0]  #入れ物の入れ替え(3次元から1次元へ)
Tw_max = Tw_max[:,0,0]  #入れ物の入れ替え(3次元から1次元へ)
Tw_min = Tw_min[:,0,0]  #入れ物の入れ替え(3次元から1次元へ)
Tw_mea_N = Tw_mea_N[:,0,0]  #入れ物の入れ替え(3次元から1次元へ)
Tw_max_N = Tw_max_N[:,0,0]  #入れ物の入れ替え(3次元から1次元へ)
Tw_min_N = Tw_min_N[:,0,0]  #入れ物の入れ替え(3次元から1次元へ)

# 計算結果をファイルに保存します--------------------------------
Table = np.array([Tw_mea, Tw_max, Tw_min, Tw_mea_N, Tw_max_N, Tw_min_N])
AMD.PutCSV_TS(Table, tim, header='Date, Tw_mea, Tw_max, Tw_min, Tw_mea_N, Tw_max_N, Tw_min_N')				#単純時系列のCSVファイル出力



Msh=Tw_mea     #グラフにする要素を指定(日平均水温)
MshN=Tw_mea_N  #グラフにする要素を指定(平年値から計算される日平均水温)
nam='Water Temperature' #グラフの縦軸の名前
uni='degC'              #グラフの縦軸の単位


# この部分を有効にすると、積算値のグラフを作成することができます。
"""
for i in range(len(tim)):
    Msh[i] = Msh[i] + Msh[i-1]
    MshN[i] = MshN[i] + MshN[i-1]
"""
#tim = np.array([datetime(x.year,x.month,x.day) for x in tim]) # エラーが表示される場合はこの行を有効にしてください。

#以下、基準値と比較する折れ線グラフを書くのに重宝します。必要に応じてコピペして使いまわしてください。－－－－－－－－
D1D1 = Msh       #太線で描くデータをD1D1という名の変数に入れてください。
D1D2 = MshN      #細線で描くデータをD1D2という名の変数に入れてください。
#tim = tim       #日付はtimという名の変数に入れてください。
figtitle = 'N'+str(lalodomain[0])+', E'+str(lalodomain[2])          
#－－－－－－－－－－－
D1D1 = np.array(D1D1)
D1D2 = np.array(D1D2)
tim = np.array(tim)
fig = plt.figure(num=None, figsize=(12, 4))
# ・目盛の作成
ax = plt.axes()
xmajoPos = md.DayLocator(bymonthday=[1])
xmajoFmt = md.DateFormatter('%m/%d')
ax.xaxis.set_major_locator(xmajoPos)
ax.xaxis.set_major_formatter(xmajoFmt)
xminoPos = md.DayLocator()
ax.xaxis.set_minor_locator(xminoPos)
# ・データのプロット
ax.fill_between(tim,D1D1,D1D2,where=D1D1>D1D2,facecolor='orange',alpha=0.5,interpolate=True)	   #細線より高い部分を橙色に塗る
ax.fill_between(tim,D1D2,D1D1,where=D1D1<D1D2,facecolor='skyblue',alpha=0.5,interpolate=True)   #細線より低い部分を水色に塗る
ax.plot(tim, D1D1, 'k')						#太線
ax.plot(tim, D1D2, 'k', linewidth=0.3)	#細線
# ・「今日」印を付ける
p = datetime.today()							      			#今日の時刻オブジェクト
today = tim == datetime(p.year,p.month,p.day,0,0,0)		#今日の配列要素番号
plt.plot(tim[today], D1D1[today], "ro")						#今日に赤点を打つ
# ・ラベル、タイトルの付加
plt.xlabel('Date')
plt.ylabel(nam+' [' + uni + ']')
plt.title(figtitle)
plt.savefig('result'+'.png', dpi=600) #この文をコメントアウトすると、図のpngファイルは作られません。
plt.show()
plt.clf()

